@@ -1,39 +1,85 @@
+from typing import Callable, List, Union
+
 from ..utils import common_functions as c_f
 from .base import BaseConditionHook, BaseHook, BaseWrapperHook
 
 
 class EmptyHook(BaseHook):
+    """Returns two empty dictionaries."""
+
     def call(self, losses, inputs):
+        """"""
         return {}, {}
 
     def _loss_keys(self):
+        """"""
         return []
 
     def _out_keys(self):
+        """"""
         return []
 
 
 class ZeroLossHook(BaseHook):
-    def __init__(self, loss_names, out_names, **kwargs):
+    """
+    Returns only 0 losses and ```None``` outputs.
+    """
+
+    def __init__(self, loss_names: List[str], out_names: List[str], **kwargs):
+        """
+        Arguments:
+            loss_names: The keys of the loss dictionary
+                which will have ```tensor(0.)``` as its values.
+            out_names: The keys of the output dictionary
+                which will have ```None``` as its values.
+        """
         super().__init__(**kwargs)
         self.loss_names = loss_names
         self.out_names = out_names
 
     def call(self, losses, inputs):
+        """"""
         out_keys = set(self.out_names) - inputs.keys()
         return {k: c_f.zero_loss() for k in self.loss_names}, {
             k: None for k in out_keys
         }
 
     def _loss_keys(self):
+        """"""
         return self.loss_names
 
     def _out_keys(self):
+        """"""
         return self.out_names
 
 
 class ChainHook(BaseHook):
-    def __init__(self, *hooks, conditions=None, alts=None, overwrite=False, **kwargs):
+    """
+    Calls multiple hooks sequentially.
+    The Nth hook receives the context accumulated through hooks 0 to N-1.
+    """
+
+    def __init__(
+        self,
+        *hooks: BaseHook,
+        conditions: List[BaseConditionHook] = None,
+        alts: List[BaseHook] = None,
+        overwrite: Union[bool, List[int]] = False,
+        **kwargs,
+    ):
+        """
+        Arguments:
+            hooks: a sequence of hooks that will be called sequentially.
+            conditions: an optional list of condition hooks.
+                If conditions[i] returns False, then alts[i] is called. Otherwise hooks[i] is called.
+            alts: an optional list of hooks that will be executed
+                when the corresponding condition hook returns False
+            overwrite: If True, then hooks will be allowed to overwrite keys in the context.
+                If a list of integers, then the hooks at the specified indices
+                will be allowed to overwrite keys in the context.
+
+        """
+
         super().__init__(**kwargs)
         self.hooks = hooks
         self.conditions = c_f.default(
@@ -49,6 +95,7 @@ class ChainHook(BaseHook):
         self.in_keys = self.hooks[0].in_keys
 
     def call(self, losses, inputs):
+        """"""
         losses, outputs = {}, {}
         all_inputs = inputs
         prev_outputs = {}
@@ -82,9 +129,11 @@ class ChainHook(BaseHook):
         self.check_overlap(losses, new_losses, ["losses", "new_losses"])
 
     def _loss_keys(self):
+        """"""
         return c_f.join_lists([h.loss_keys for h in self.hooks])
 
     def _out_keys(self):
+        """"""
         return c_f.join_lists([h.out_keys for h in self.hooks])
 
     @property
@@ -113,12 +162,24 @@ class ChainHook(BaseHook):
 # outputs are not shared among parallel streams
 # they are returned at the very end
 class ParallelHook(BaseHook):
-    def __init__(self, *hooks, **kwargs):
+    """
+    Calls multiple hooks while keeping contexts separate.
+    The Nth hook receives the same context as hooks 0 to N-1.
+    All the output contexts are merged at the end.
+    """
+
+    def __init__(self, *hooks: BaseHook, **kwargs):
+        """
+        Arguments:
+            hooks: a sequence of hooks that will be called sequentially,
+                with each hook receiving the same initial context.
+        """
         super().__init__(**kwargs)
         self.hooks = hooks
         self.in_keys = c_f.join_lists([h.in_keys for h in self.hooks])
 
     def call(self, losses, inputs):
+        """"""
         losses, outputs = {}, {}
         for h in self.hooks:
             x = h(losses, inputs)
@@ -133,35 +194,59 @@ class ParallelHook(BaseHook):
         return x
 
     def _loss_keys(self):
+        """"""
         return c_f.join_lists([h.loss_keys for h in self.hooks])
 
     def _out_keys(self):
+        """"""
         return c_f.join_lists([h.out_keys for h in self.hooks])
 
 
-# Returns only outputs that are not present in kwargs
-# You should use this if you want to change the value of
-# a key passed to self.hook, but not propagate that change
-# to the outside
 class OnlyNewOutputsHook(BaseWrapperHook):
-    def __init__(self, hook, **kwargs):
+    """
+    Returns only outputs that are not present in the input context.
+    You should use this if you want to change the value of
+    a key passed to self.hook, but not propagate that change
+    to the outside.
+    """
+
+    def __init__(self, hook: BaseHook, **kwargs):
+        """
+        Arguments:
+            hook: The hook inside which changes to the context will be allowed.
+        """
         super().__init__(**kwargs)
         self.hook = hook
 
     def call(self, losses, inputs):
+        """"""
         losses, outputs = self.hook(losses, inputs)
         outputs = {k: outputs[k] for k in (outputs.keys() - inputs.keys())}
         return losses, outputs
 
 
 class ApplyFnHook(BaseHook):
-    def __init__(self, fn, apply_to, is_loss=False, **kwargs):
+    """
+    Applies a function to specific values of the context.
+    """
+
+    def __init__(
+        self, fn: Callable, apply_to: List[str], is_loss: bool = False, **kwargs
+    ):
+        """
+        Arguments:
+            fn: The function that will be applied to the inputs.
+            apply_to: fn will be applied to ```inputs[k]``` for k in apply_to
+            is_loss: If False, then the returned loss dictionary will be empty.
+                Otherwise, the returned output dictionary will be empty.
+        """
         super().__init__(**kwargs)
         self.fn = fn
         self.apply_to = apply_to
         self.is_loss = is_loss
 
     def call(self, losses, inputs):
+        """"""
         x = c_f.extract(inputs, self.apply_to)
         outputs = {k: self.fn(v) for k, v in zip(self.apply_to, x)}
         if self.is_loss:
@@ -169,9 +254,11 @@ class ApplyFnHook(BaseHook):
         return {}, outputs
 
     def _loss_keys(self):
+        """"""
         return self.apply_to if self.is_loss else []
 
     def _out_keys(self):
+        """"""
         return [] if self.is_loss else self.apply_to
 
     def extra_repr(self):
@@ -179,26 +266,49 @@ class ApplyFnHook(BaseHook):
 
 
 class TrueHook(BaseConditionHook):
+    """Returns ```True```"""
+
     def call(self, losses, inputs):
+        """"""
         return True
 
 
 class FalseHook(BaseConditionHook):
+    """Returns ```False```"""
+
     def call(self, losses, inputs):
+        """"""
         return False
 
 
 class NotHook(BaseConditionHook):
-    def __init__(self, hook, **kwargs):
+    """Returns the boolean negation of the wrapped hook."""
+
+    def __init__(self, hook: BaseConditionHook, **kwargs):
+        """
+        Arguments:
+            hook: The condition hook that will be negated.
+        """
         super().__init__(**kwargs)
         self.hook = hook
 
     def call(self, losses, inputs):
+        """"""
         return not self.hook(losses, inputs)
 
 
 class AssertHook(BaseWrapperHook):
-    def __init__(self, hook, allowed, **kwargs):
+    """
+    Asserts that the output keys of a hook match a specified regex string
+    """
+
+    def __init__(self, hook: BaseHook, allowed: str, **kwargs):
+        """
+        Arguments:
+            hook: The wrapped hook
+            allowed: The output dictionary of ```hook```
+                must have keys that match the ```allowed``` regex.
+        """
         super().__init__(**kwargs)
         self.hook = hook
         if not isinstance(allowed, str):
@@ -206,6 +316,7 @@ class AssertHook(BaseWrapperHook):
         self.allowed = allowed
 
     def call(self, losses, inputs):
+        """"""
         losses, outputs = self.hook(losses, inputs)
         self.assert_fn(outputs)
         return losses, outputs
@@ -223,12 +334,22 @@ class AssertHook(BaseWrapperHook):
 
 
 class MultiplierHook(BaseWrapperHook):
-    def __init__(self, hook, m, **kwargs):
+    """
+    Multiplies every loss by a scalar
+    """
+
+    def __init__(self, hook: BaseHook, m: float, **kwargs):
+        """
+        Arguments:
+            hook: The losses of this hook will be multiplied by ```m```
+            m: The scalar
+        """
         super().__init__(**kwargs)
         self.hook = hook
         self.m = m
 
     def call(self, losses, inputs):
+        """"""
         losses, outputs = self.hook(losses, inputs)
         losses = {k: v * self.m for k, v in losses.items()}
         return losses, outputs
@@ -238,13 +359,27 @@ class MultiplierHook(BaseWrapperHook):
 
 
 class RepeatHook(BaseHook):
-    def __init__(self, hook, n, keep_only_last=False, **kwargs):
+    """
+    Executes the wrapped hook ```n``` times.
+    """
+
+    def __init__(self, hook: BaseHook, n: int, keep_only_last: bool = False, **kwargs):
+        """
+        Arguments:
+            hook: The hook that will be executed ```n``` times
+            n: The number of times the hook will be executed.
+            keep_only_last: If ```False```, the (losses, outputs) from each execution
+                will be accumulated, and the keys will have the iteration number appended.
+                If ```True```, then only the (losses, outputs) of the final execution will
+                be kept.
+        """
         super().__init__(**kwargs)
         self.hook = hook
         self.n = n
         self.keep_only_last = keep_only_last
 
     def call(self, losses, inputs):
+        """"""
         losses, outputs = {}, {}
         for i in range(self.n):
             x = self.hook(losses, inputs)
@@ -256,12 +391,14 @@ class RepeatHook(BaseHook):
         return losses, outputs
 
     def _loss_keys(self):
+        """"""
         if self.keep_only_last:
             return self.hook.loss_keys
         else:
             return [f"{k}{i}" for k in self.hook.loss_keys for i in range(self.n)]
 
     def _out_keys(self):
+        """"""
         if self.keep_only_last:
             return self.hook.out_keys
         else:
@@ -279,6 +416,7 @@ class ApplyToListHook(BaseWrapperHook):
         self.regex = regex
 
     def call(self, losses, inputs):
+        """"""
         search_strs = c_f.filter(inputs, self.regex)
         extracted = c_f.extract(inputs, search_strs)
         if len(set(len(x) for x in extracted)) != 1:
@@ -292,9 +430,11 @@ class ApplyToListHook(BaseWrapperHook):
         return losses, {}
 
     def _loss_keys(self):
+        """"""
         return [f"{k}{i}" for k in self.hook.loss_keys for i in range(self.list_size)]
 
     def _out_keys(self):
+        """"""
         return []
 
     def extra_repr(self):
